# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**Version**: AceCommon v1.5.2

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`).

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

**v1.4.5**

* Upgrading from ESP32 Core v1.0.4 to v1.0.6 causes `PrintStrN` to be about
  1300 bytes larger than `PrintStr`, probably because the `new` and `delete`
  operators bring in more complex code. But the overall ESP32 flash usage goes
  down by 7.5 kB compared to v1.0.4, so this is probably not something to worry
  about.
* Added benchmarks for `udiv1000()` and native `/1000`. The `udiv1000()`
  function consumes about 180 bytes on 8-bit AVR processors, more than the
  native division by `/1000` which takes about 130 bytes.

**v1.4.6**

* Upgrade STM32duino Core from 1.9.0 to 2.0.0.
    * Flash memory increases by 2.3kB across the board.
    * Static memory decreases by 250 bytes across the board.
    * AceCommon code unchanged.
* Upgrade SparkFun SAMD Core from 1.8.1 to 1.8.3.
    * No changes to flash or static memory.
* Add benchmark for `String` so to compare with `PrintStr<N>` and `PrintStrN`.
    * Pull in `Serial` into the Baseline to compensate for pulling in the
      `Print` class, the parent class of `PrintStr` and `PrintStrN`.
* Add benchmark for `isSorted()`.
    * Add `array[100]` into Baseline to compensate for its use in `isSorted()`.
* Add benchmark for `printUint16AsFloat3To()` and `printUint32AsFloat3To()`.

**v1.4.7**

* Add benchmarks for `KString::compareTo(const KString&)`.
* Upgrade Arduino IDE from 1.8.13 to 1.8.16.
* Upgrade Arduino CLI from 0.14.0 to 0.19.2
* Upgrade SparkFun SAMD Core from 1.8.3 to 1.8.5.
* Upgrade ESP8266 Core from 2.7.4 to 3.0.2.
* Upgrade Teensyduino from 1.53 to 1.55.

**v1.5.0**

* Remove SAMD21 board.
* Add benchmark for `backslashXEncode()` and `backslashXDecode()`.
* Upgrade Arduino IDE from 1.8.16 to 1.8.19.
* Upgrade Arduino CLI from 0.19.2 to 0.20.2.
* Upgrade Arduino AVR Core from 1.8.3 to 1.8.4.
* Upgrade STM32 Core from 2.0.0 to 2.2.0.
* Upgrade ESP32 Core from 1.0.6 to 2.0.2.
* Upgrade Teensyduino from 1.55 to 1.56.

**v1.5.2**

* Upgrade tool chain
    * Upgrade Arduino CLI from 0.20.2 to 0.27.1.
    * Upgrade Arduino AVR Core from 1.8.4 to 1.8.5.
    * Upgrade STM32 Core from 2.2.0 to 2.3.0.
    * Upgrade ESP32 Core from 2.0.2 to 2.0.5.
    * Upgrade Teensyduino from 1.56 to 1.57.
* No significant changes to memory sizes.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* Arduino AVR Boards 1.8.5

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   2146/  289 |     0/    0 |
|----------------------------------------+--------------+-------------|
| String                                 |   3620/  301 |  1474/   12 |
| PrintStr<16>                           |   2520/  301 |   374/   12 |
| PrintStrN(16)                          |   3124/  311 |   978/   22 |
|----------------------------------------+--------------+-------------|
| printPad2()                            |   2230/  289 |    84/    0 |
| printPad5()                            |   2248/  289 |   102/    0 |
| printUint16AsFloat3To()                |   2242/  289 |    96/    0 |
| printUint32AsFloat3To()                |   2246/  289 |   100/    0 |
|----------------------------------------+--------------+-------------|
| printReplaceCharTo(char*)              |   2186/  293 |    40/    4 |
| printReplaceCharTo(F())                |   2182/  289 |    36/    0 |
| printReplaceStringTo(char*)            |   2204/  295 |    58/    6 |
| printReplaceStringTo(F())              |   2200/  291 |    54/    2 |
|----------------------------------------+--------------+-------------|
| hashDjb2(char*)                        |   2216/  295 |    70/    6 |
| hashDjb2(F())                          |   2212/  289 |    66/    0 |
|----------------------------------------+--------------+-------------|
| udiv1000()                             |   2282/  289 |   136/    0 |
| /1000                                  |   2216/  289 |    70/    0 |
|----------------------------------------+--------------+-------------|
| isSorted()                             |   2182/  289 |    36/    0 |
| reverse()                              |   2178/  289 |    32/    0 |
|----------------------------------------+--------------+-------------|
| KString::compareTo(char*)              |   2270/  317 |   124/   28 |
| KString::compareTo(KString&)           |   2516/  311 |   370/   22 |
|----------------------------------------+--------------+-------------|
| backslashXEncode()                     |   2308/  295 |   162/    6 |
| backslashXDecode()                     |   2364/  301 |   218/   12 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   4268/  254 |     0/    0 |
|----------------------------------------+--------------+-------------|
| String                                 |   5742/  264 |  1474/   10 |
| PrintStr<16>                           |   4636/  266 |   368/   12 |
| PrintStrN(16)                          |   5246/  276 |   978/   22 |
|----------------------------------------+--------------+-------------|
| printPad2()                            |   4352/  254 |    84/    0 |
| printPad5()                            |   4368/  254 |   100/    0 |
| printUint16AsFloat3To()                |   4364/  254 |    96/    0 |
| printUint32AsFloat3To()                |   4368/  254 |   100/    0 |
|----------------------------------------+--------------+-------------|
| printReplaceCharTo(char*)              |   4308/  258 |    40/    4 |
| printReplaceCharTo(F())                |   4304/  254 |    36/    0 |
| printReplaceStringTo(char*)            |   4326/  260 |    58/    6 |
| printReplaceStringTo(F())              |   4322/  256 |    54/    2 |
|----------------------------------------+--------------+-------------|
| hashDjb2(char*)                        |   4336/  258 |    68/    4 |
| hashDjb2(F())                          |   4334/  254 |    66/    0 |
|----------------------------------------+--------------+-------------|
| udiv1000()                             |   4404/  254 |   136/    0 |
| /1000                                  |   4338/  254 |    70/    0 |
|----------------------------------------+--------------+-------------|
| isSorted()                             |   4304/  254 |    36/    0 |
| reverse()                              |   4300/  254 |    32/    0 |
|----------------------------------------+--------------+-------------|
| KString::compareTo(char*)              |   4392/  282 |   124/   28 |
| KString::compareTo(KString&)           |   4638/  276 |   370/   22 |
|----------------------------------------+--------------+-------------|
| backslashXEncode()                     |   4430/  260 |   162/    6 |
| backslashXDecode()                     |   4502/  266 |   234/   12 |
+---------------------------------------------------------------------+

```

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* STM32duino 2.3.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  27144/ 3776 |     0/    0 |
|----------------------------------------+--------------+-------------|
| String                                 |  27696/ 3776 |   552/    0 |
| PrintStr<16>                           |  27304/ 3776 |   160/    0 |
| PrintStrN(16)                          |  27320/ 3776 |   176/    0 |
|----------------------------------------+--------------+-------------|
| printPad2()                            |  27240/ 3776 |    96/    0 |
| printPad5()                            |  27264/ 3776 |   120/    0 |
| printUint16AsFloat3To()                |  27248/ 3776 |   104/    0 |
| printUint32AsFloat3To()                |  27252/ 3776 |   108/    0 |
|----------------------------------------+--------------+-------------|
| printReplaceCharTo(char*)              |  27180/ 3776 |    36/    0 |
| printReplaceCharTo(F())                |  27180/ 3776 |    36/    0 |
| printReplaceStringTo(char*)            |  27200/ 3776 |    56/    0 |
| printReplaceStringTo(F())              |  27200/ 3776 |    56/    0 |
|----------------------------------------+--------------+-------------|
| hashDjb2(char*)                        |  27176/ 3776 |    32/    0 |
| hashDjb2(F())                          |  27176/ 3776 |    32/    0 |
|----------------------------------------+--------------+-------------|
| udiv1000()                             |  27184/ 3776 |    40/    0 |
| /1000                                  |  27176/ 3776 |    32/    0 |
|----------------------------------------+--------------+-------------|
| isSorted()                             |  27180/ 3776 |    36/    0 |
| reverse()                              |  27176/ 3776 |    32/    0 |
|----------------------------------------+--------------+-------------|
| KString::compareTo(char*)              |  27316/ 3776 |   172/    0 |
| KString::compareTo(KString&)           |  27388/ 3776 |   244/    0 |
|----------------------------------------+--------------+-------------|
| backslashXEncode()                     |  27356/ 3776 |   212/    0 |
| backslashXDecode()                     |  27392/ 3776 |   248/    0 |
+---------------------------------------------------------------------+

```

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* ESP8266 Boards 3.0.2

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 265613/28104 |     0/    0 |
|----------------------------------------+--------------+-------------|
| String                                 | 265713/28100 |   100/   -4 |
| PrintStr<16>                           | 265821/28104 |   208/    0 |
| PrintStrN(16)                          | 265921/28104 |   308/    0 |
|----------------------------------------+--------------+-------------|
| printPad2()                            | 265773/28104 |   160/    0 |
| printPad5()                            | 265805/28104 |   192/    0 |
| printUint16AsFloat3To()                | 265789/28104 |   176/    0 |
| printUint32AsFloat3To()                | 265805/28104 |   192/    0 |
|----------------------------------------+--------------+-------------|
| printReplaceCharTo(char*)              | 265665/28100 |    52/   -4 |
| printReplaceCharTo(F())                | 265665/28104 |    52/    0 |
| printReplaceStringTo(char*)            | 265701/28104 |    88/    0 |
| printReplaceStringTo(F())              | 265717/28100 |   104/   -4 |
|----------------------------------------+--------------+-------------|
| hashDjb2(char*)                        | 265669/28104 |    56/    0 |
| hashDjb2(F())                          | 265685/28104 |    72/    0 |
|----------------------------------------+--------------+-------------|
| udiv1000()                             | 265677/28104 |    64/    0 |
| /1000                                  | 265677/28104 |    64/    0 |
|----------------------------------------+--------------+-------------|
| isSorted()                             | 265645/28104 |    32/    0 |
| reverse()                              | 265661/28104 |    48/    0 |
|----------------------------------------+--------------+-------------|
| KString::compareTo(char*)              | 265817/28132 |   204/   28 |
| KString::compareTo(KString&)           | 265941/28120 |   328/   16 |
|----------------------------------------+--------------+-------------|
| backslashXEncode()                     | 265877/28104 |   264/    0 |
| backslashXDecode()                     | 265929/28116 |   316/   12 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* ESP32 Boards 2.0.5

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 241753/16480 |     0/    0 |
|----------------------------------------+--------------+-------------|
| String                                 | 242109/16480 |   356/    0 |
| PrintStr<16>                           | 242017/16480 |   264/    0 |
| PrintStrN(16)                          | 242333/16480 |   580/    0 |
|----------------------------------------+--------------+-------------|
| printPad2()                            | 241849/16480 |    96/    0 |
| printPad5()                            | 241869/16480 |   116/    0 |
| printUint16AsFloat3To()                | 241857/16480 |   104/    0 |
| printUint32AsFloat3To()                | 241877/16480 |   124/    0 |
|----------------------------------------+--------------+-------------|
| printReplaceCharTo(char*)              | 241781/16480 |    28/    0 |
| printReplaceCharTo(F())                | 241781/16480 |    28/    0 |
| printReplaceStringTo(char*)            | 241817/16480 |    64/    0 |
| printReplaceStringTo(F())              | 241817/16480 |    64/    0 |
|----------------------------------------+--------------+-------------|
| hashDjb2(char*)                        | 241789/16480 |    36/    0 |
| hashDjb2(F())                          | 241789/16480 |    36/    0 |
|----------------------------------------+--------------+-------------|
| udiv1000()                             | 241833/16480 |    80/    0 |
| /1000                                  | 241817/16480 |    64/    0 |
|----------------------------------------+--------------+-------------|
| isSorted()                             | 241785/16480 |    32/    0 |
| reverse()                              | 241789/16480 |    36/    0 |
|----------------------------------------+--------------+-------------|
| KString::compareTo(char*)              | 241917/16480 |   164/    0 |
| KString::compareTo(KString&)           | 242017/16480 |   264/    0 |
|----------------------------------------+--------------+-------------|
| backslashXEncode()                     | 241973/16480 |   220/    0 |
| backslashXDecode()                     | 242021/16480 |   268/    0 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.19, Arduino CLI 0.27.1
* Teensyduino 1.57

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  11264/ 4276 |     0/    0 |
|----------------------------------------+--------------+-------------|
| String                                 |  13772/ 4276 |  2508/    0 |
| PrintStr<16>                           |  11480/ 4276 |   216/    0 |
| PrintStrN(16)                          |  11516/ 4276 |   252/    0 |
|----------------------------------------+--------------+-------------|
| printPad2()                            |  11292/ 4276 |    28/    0 |
| printPad5()                            |  11312/ 4276 |    48/    0 |
| printUint16AsFloat3To()                |  11300/ 4276 |    36/    0 |
| printUint32AsFloat3To()                |  11304/ 4276 |    40/    0 |
|----------------------------------------+--------------+-------------|
| printReplaceCharTo(char*)              |  11296/ 4276 |    32/    0 |
| printReplaceCharTo(F())                |  11296/ 4276 |    32/    0 |
| printReplaceStringTo(char*)            |  11316/ 4276 |    52/    0 |
| printReplaceStringTo(F())              |  11316/ 4276 |    52/    0 |
|----------------------------------------+--------------+-------------|
| hashDjb2(char*)                        |  11300/ 4276 |    36/    0 |
| hashDjb2(F())                          |  11300/ 4276 |    36/    0 |
|----------------------------------------+--------------+-------------|
| udiv1000()                             |  11304/ 4276 |    40/    0 |
| /1000                                  |  11300/ 4276 |    36/    0 |
|----------------------------------------+--------------+-------------|
| isSorted()                             |  11288/ 4276 |    24/    0 |
| reverse()                              |  11292/ 4276 |    28/    0 |
|----------------------------------------+--------------+-------------|
| KString::compareTo(char*)              |  11456/ 4276 |   192/    0 |
| KString::compareTo(KString&)           |  11500/ 4276 |   236/    0 |
|----------------------------------------+--------------+-------------|
| backslashXEncode()                     |  11508/ 4276 |   244/    0 |
| backslashXDecode()                     |  11528/ 4276 |   264/    0 |
+---------------------------------------------------------------------+

```

